﻿/*
*	FILE			: DBT.cs
*	PROJECT			: DataBasetransfer
*	PROGRAMMER		: BRIAN HINDS
*	FIRST VERSION	: 1.0.0.1
*	DESCRIPTION 	: Transfers tables between database such as SQL Server and Microsoft Access.
*/

using System;
using System.Windows.Forms;

namespace DataBaseTransfer
{
    public partial class DBT : Form
    {
        /// <summary>
        /// Connection Variables and Objects
        /// </summary>
        private string SourceConnection = "";

        private string DestinationConnection = "";

        /// <summary>
        /// Instantiating the connection manager
        /// </summary>
        private ConnectionManager CM = new ConnectionManager();

        /// <summary>
        /// Instantiating the Data base transfer manager
        /// </summary>
        private DataBaseTransferManager SQLServer = new DataBaseTransferManager();

        /// <summary>
        /// Variables for logics and log window
        /// </summary>
        private int CHKStatA = 0;

        private int CHKStatB = 0;
        private int CHKStatC = 0;
        private DateTime dt = DateTime.Now;

        public DBT()
        {
            InitializeComponent();

            btnTransfer.Enabled = false;
        }

        /// <summary>
        /// Allow user to use data link property dialog to set
        /// source connection string
        /// </summary>
        /// <method="Source_Connection"></method>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Source_Connection_Click(object sender, EventArgs e)
        {
            FormUpdater.Start();

            SourceConnection = CM.ConnectionStringWizard();
        }

        /// <summary>
        /// Allow user to user data link property dialog to set
        /// destination connection string
        /// </summary>
        /// <method="Destination_Connection"></method>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Destination_Connection_Click(object sender, EventArgs e)
        {
            DestinationConnection = CM.ConnectionStringWizard();
        }

        /// <summary>
        /// This method launches the help guide
        /// </summary>
        /// <method="HelpBtn"></method>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void HelpBtn_Click(object sender, EventArgs e)
        {
            AboutDBT help = new AboutDBT();
            help.ShowDialog();
        }

        /// <summary>
        /// This Method perform the data base transfer
        /// process
        /// </summary>
        /// <method="btnTransfer"></method>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnTransfer_Click(object sender, EventArgs e)
        {
            CHKStatA = 0; CHKStatB = 0; CHKStatC = 0;

            resetStatus();

            CHKStatA = SQLServer.destinastionTableSchema(txtTableName.Text, DestinationConnection);

            if (CHKStatA == 0)
            {
                LogWin.Items.Add(DateTime.Now.ToString("HH:mm:ss ") + "Destination table [" + txtTableName.Text + "] do not exist");
                CHKStatB = SQLServer.TableSchemaCopy(txtTableName.Text, SourceConnection, DestinationConnection);
            }

            if (CHKStatA == 1) { LogWin.Items.Add(DateTime.Now.ToString("HH:mm:ss ") + "Destination table [" + txtTableName.Text + "] found"); }

            CreateStatus();

            if (CHKStatA == 0 && CHKStatB == 2 || CHKStatA == 1)
            {
                LogWin.Items.Add(DateTime.Now.ToString("HH:mm:ss ") + "Transfer Started");
                CHKStatC = SQLServer.CopyTable(SourceConnection, DestinationConnection, txtTableName.Text);
            }

            CopyStatus();

            FormUpdater.Stop();
        }

        /// <summary>
        /// This method check until transfer is possible
        /// </summary>
        /// <method="FormUpdater"></method>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FormUpdater_Tick(object sender, EventArgs e)
        {
            if (SourceConnection != "" & DestinationConnection != "" & !String.IsNullOrEmpty(txtTableName.Text))
            { btnTransfer.Enabled = true; }
            else
            {
                btnTransfer.Enabled = false;
            }
        }

        /// <summary>
        /// setting the status icon for the process
        /// </summary>
        /// <method="CopyStatus"></method>

        private void CopyStatus()
        {
            if (CHKStatC == 1)
            {
                step1.Image = Properties.Resources.error;
                step3.Image = Properties.Resources.error;
                step2.Image = Properties.Resources.error;
                step4.Image = Properties.Resources.error;
                step5.Image = Properties.Resources.error;
                LogWin.Items.Add(DateTime.Now.ToString("HH:mm:ss ") + "Source Connection Error");
            }
            if (CHKStatC == 2)
            {
                step1.Image = Properties.Resources.pass;
                step3.Image = Properties.Resources.error;
                step2.Image = Properties.Resources.error;
                step4.Image = Properties.Resources.error;
                step5.Image = Properties.Resources.error;
                LogWin.Items.Add(DateTime.Now.ToString("HH:mm:ss ") + "Error Copying Source [" + txtTableName.Text + "] Table");
            }
            if (CHKStatC == 3)
            {
                step1.Image = Properties.Resources.pass;
                step3.Image = Properties.Resources.pass;
                step2.Image = Properties.Resources.error;
                step4.Image = Properties.Resources.error;
                step5.Image = Properties.Resources.error;
                LogWin.Items.Add(DateTime.Now.ToString("HH:mm:ss ") + "Destination Connection Error");
            }
            if (CHKStatC == 4)
            {
                step1.Image = Properties.Resources.pass;
                step3.Image = Properties.Resources.pass;
                step2.Image = Properties.Resources.pass;
                step4.Image = Properties.Resources.error;
                step5.Image = Properties.Resources.error;
                LogWin.Items.Add(DateTime.Now.ToString("HH:mm:ss ") + "Error Updating Destination Table [" + txtTableName.Text + "] [one or more fields may not allow duplicates]");
            }
            if (CHKStatC == 5)
            {
                step1.Image = Properties.Resources.pass;
                step3.Image = Properties.Resources.pass;
                step2.Image = Properties.Resources.pass;
                step4.Image = Properties.Resources.pass;
                step5.Image = Properties.Resources.error;
                LogWin.Items.Add(DateTime.Now.ToString("HH:mm:ss ") + "Destination Table [" + txtTableName.Text + "] Update With Error ");
            }
            if (CHKStatC == 6)
            {
                step1.Image = Properties.Resources.pass;
                step3.Image = Properties.Resources.pass;
                step2.Image = Properties.Resources.pass;
                step4.Image = Properties.Resources.pass;
                step5.Image = Properties.Resources.pass;
                LogWin.Items.Add(DateTime.Now.ToString("HH:mm:ss ") + "[" + txtTableName.Text + "] Table Copied To Destination Successfully");
            }
            LogWin.Items.Add(" ");
        }

        /// <summary>
        ///
        /// </summary>
        private void CreateStatus()
        {
            if (CHKStatB == 1)
            {
                StepA.Image = Properties.Resources.error;
                StepB.Image = Properties.Resources.error;
                LogWin.Items.Add(DateTime.Now.ToString("HH:mm:ss ") + "Table Failed to be created, check table name, or source table don't exist");
            }
            if (CHKStatB == 2)
            {
                StepA.Image = Properties.Resources.pass;
                StepB.Image = Properties.Resources.pass;
                LogWin.Items.Add(DateTime.Now.ToString("HH:mm:ss ") + "[" + txtTableName.Text + "] Table Created");
            }
        }

        /// <summary>
        /// Resetting status icon for process
        /// </summary>
        /// <method="resetStatus"></method>
        private void resetStatus()
        {
            step1.Image = Properties.Resources.excl2;
            step3.Image = Properties.Resources.excl2;
            step2.Image = Properties.Resources.excl2;
            step4.Image = Properties.Resources.excl2;
            step5.Image = Properties.Resources.excl2;
            StepA.Image = Properties.Resources.excl2;
            StepB.Image = Properties.Resources.excl2;
        }
    }
}