﻿/*
*	FILE			: DataBaseTransferManager.cs
*	PROJECT			: DataBasetransfer
*	PROGRAMMER		: BRIAN HINDS
*	FIRST VERSION	: 1.0.0.1
*	DESCRIPTION 	: Transfers tables between database such as SQL Server and Microsoft Access.
*/

using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Data.SqlClient;
using System.Data.OleDb;
using System.Windows.Forms;

namespace DataBaseTransfer
{
    internal class DataBaseTransferManager
    {
        private int CopytableStatus = 0;

        /// <summary>
        /// Copying SQL server tables between data base
        /// </summary>
        /// <method="CopyTable"></method>
        /// <param name="Source"></param>
        /// <param name="Destination"></param>
        /// <param name="Table"></param>
        /// <returns></returns>
        public int CopyTable(string Source, string Destination, string Table)
        {
            OleDbTransaction transaction = null;

            /// Creating a connection to the source ///
            OleDbConnection SourceDB = new OleDbConnection(Source);

            /// Creating a connection to the destination ///
            OleDbConnection DestinationDB = new OleDbConnection(Destination);

            CopytableStatus = 1;

            try
            {
                /// Opening source connection ///
                SourceDB.Open();
                CopytableStatus = 2;

                /// Begin transaction ///
                transaction = SourceDB.BeginTransaction();

                /// SQL command for the source table ///
                OleDbDataAdapter ODDA1 = new OleDbDataAdapter(new OleDbCommand("SELECT * FROM " + Table, SourceDB, transaction));

                /// Create a data table for the source data ///
                DataTable DT1 = new DataTable();

                /// Add the rows in the source data table from the source table ///
                ODDA1.Fill(DT1);

                /// Copies the structure and data from the source data table to the destination data table ///
                DataTable DT2 = DT1.Copy();

                /// inserting the data into each rows of the source data table ///
                foreach (DataRow row in DT2.Rows) { row.SetAdded(); }

                /// Gets the current row state with relation to the DataRow ///
                DataRowState DRS = DT2.Rows[0].RowState;

                CopytableStatus = 3;
                /// SQL command for the destination table ///
                OleDbDataAdapter ODDA2 = new OleDbDataAdapter(new OleDbCommand("SELECT * FROM " + Table, DestinationDB));

                /// Building the destination command to execute ///
                OleDbCommandBuilder ODCB = new OleDbCommandBuilder(ODDA2);

                /// Opening Destination connection ///
                DestinationDB.Open();

                CopytableStatus = 4;

                /// Updating the value in the destination database by executing a update statement ///
                ODDA2.Update(DT2);
                CopytableStatus = 5;

                // Committing transaction ///
                transaction.Commit();

                /// Close source connection ///
                SourceDB.Close();

                /// Close destination connection
                DestinationDB.Close();

                CopytableStatus = 6;
            }
            catch (Exception)
            {
                /// Rollback transaction ///
                transaction.Rollback();

                return CopytableStatus;
            }

            return CopytableStatus;
        }

        /// <summary>
        /// This method creates a table schema from the source
        /// data table from the specified table of the user
        /// </summary>
        /// <method="TableSchema"></method>
        /// <param name="Table"></param>
        /// <param name="Source"></param>
        /// <returns>Status</returns>
        public int TableSchemaCopy(string Table, string Source, string destination)
        {
            /// Creating a connection to the source ///
            OleDbConnection SourceDB = new OleDbConnection(Source);

            try
            {
                OleDbDataAdapter OLDA = new OleDbDataAdapter("SELECT * FROM " + Table, SourceDB);

                /// Creating data table ///
                DataTable DT = new DataTable(Table);

                /// Getting table schema ///
                OLDA.FillSchema(DT, SchemaType.Source);

                /// Attempt to create a table in the destination database ///
                CreateDestinationTable(DT, destination, Table);
            }
            catch (Exception)
            {
                return 1;
            }

            return 2;
        }

        /// <summary>
        /// This Method use the source data table schema to
        /// create a table in the destination with the same
        /// properties
        /// </summary>
        /// <param name="DT"></param>
        /// <param name="CreateDestinationTable"></param>
        /// <param name="Table"></param>
        private void CreateDestinationTable(DataTable DT, string Destination, string Table)
        {
            StringBuilder SQL_Command = new StringBuilder();

            /// starting the building of the table string ///
            SQL_Command.Append("CREATE TABLE [" + Table + "] (" + Environment.NewLine);

            /// Cycle through the column collection of the source table ///
            foreach (DataColumn DC in DT.Columns)
            {
                /// Adding the column ///
                SQL_Command.Append("[" + DC.ColumnName + "] ");

                /// Mapping the source column and converting to a SQL server syntax ///
                SQL_Command.Append(Net_to_SQL(DC.DataType.ToString(), DC.MaxLength) + " ");

                /// Adding the identity information ///
                if (DC.AutoIncrement) { SQL_Command.Append("IDENTITY "); }

                /// Adding the AllowNull information ///
                SQL_Command.Append((DC.AllowDBNull ? "" : "NOT ") + "NULL, " + Environment.NewLine);
            }

            SQL_Command.Remove(SQL_Command.Length - (Environment.NewLine.Length + 1), 1);

            SQL_Command.Append(") ON [PRIMARY];" + Environment.NewLine + Environment.NewLine);

            /// Adding a primary key to the table if it already exists ///
            if (DT.PrimaryKey != null)
            {
                SQL_Command.Append("ALTER TABLE " + Table + " WITH NOCHECK ADD " + Environment.NewLine);

                SQL_Command.Append("CONSTRAINT [PK_" + Table + "] PRIMARY KEY CLUSTERED (" + Environment.NewLine);

                /// Adding the column to the primary key ///
                foreach (DataColumn DC in DT.PrimaryKey)
                {
                    SQL_Command.Append("[" + DC.ColumnName + "]," + Environment.NewLine);
                }

                SQL_Command.Remove(SQL_Command.Length - (Environment.NewLine.Length + 1), 1);

                SQL_Command.Append(") ON [PRIMARY];" + Environment.NewLine + Environment.NewLine);

                OleDbConnection DestinationDB = new OleDbConnection(Destination);

                try
                {
                    /// SQL command to execute and create the table in the destination database ///
                    OleDbCommand Command = new OleDbCommand(SQL_Command.ToString(), DestinationDB);

                    DestinationDB.Open();

                    Command.ExecuteNonQuery();

                    DestinationDB.Close();
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.ToString(), "Create Destination Table Failed", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        /// <summary>
        /// This method is use to map the .Net data types
        /// to the SQL Server types.
        /// </summary>
        /// <method="Net_to_SQL"></method>
        /// <param name="NetType"></param>
        /// <param name="MaxLenght"></param>
        /// <returns>SQLType</returns>
        private string Net_to_SQL(string NetType, int MaxLenght)
        {
            String sqlType = "";

            switch (NetType)
            {
                case "System.Boolean":
                    sqlType = "[bit]";
                    break;

                case "System.Byte":
                    sqlType = "[tinyint]";
                    break;

                case "System.Int16":
                    sqlType = "[smallint]";
                    break;

                case "System.Int32":
                    sqlType = "[int]";
                    break;

                case "System.Int64":
                    sqlType = "[bigint]";
                    break;

                case "System.Byte[]":
                    sqlType = "[binary]";
                    break;

                case "System.Char[]":
                    sqlType = "[nchar] (" + MaxLenght + ")";
                    break;

                case "System.String":
                    if (MaxLenght == 0x3FFFFFFF)
                        sqlType = "[ntext]";
                    else
                        sqlType = "[nvarchar] (" + MaxLenght + ")";
                    break;

                case "System.Single":
                    sqlType = "[real]";
                    break;

                case "System.Double":
                    sqlType = "[float]";
                    break;

                case "System.Decimal":
                    sqlType = "[decimal]";
                    break;

                case "System.DateTime":
                    sqlType = "[datetime]";
                    break;

                case "System.Guid":
                    sqlType = "[uniqueidentifier]";
                    break;

                case "System.Object":
                    sqlType = "[sql_variant]";
                    break;
            }

            return sqlType;
        }

        /// <summary>
        /// Checks for a destination table
        /// </summary>
        /// <param name="Table"></param>
        /// <param name="destination"></param>
        /// <returns></returns>
        public int destinastionTableSchema(string Table, string destination)
        {
            /// Creating a connection to the source ///
            OleDbConnection destinationDB = new OleDbConnection(destination);

            try
            {
                OleDbDataAdapter OLDA = new OleDbDataAdapter("SELECT * FROM " + Table, destinationDB);

                /// Creating data table ///
                DataTable DT = new DataTable(Table);

                /// Getting table schema ///
                OLDA.FillSchema(DT, SchemaType.Source);
            }
            catch (Exception)
            {
                return 0;
            }

            return 1;
        }
    }
}